### A TOY EXAMPLE WITH A SIMULATED DATASET ###
### NOTE THAT THE FOLLOWING CODE IS ONLY FOR ONE PHENOTYPE REPLICATE ###

#install.packages("matrixStats");install.packages("glmnet")
library(matrixStats)
library(glmnet)


#A simulated model matrix consisting of normally distributed expression values of 250 mutually independent genes for 200 individuals.
expset <- matrix(0,200,250)
for(i in 1:250){
  expset[,i] <- rnorm(200,0,1)
}

#Residual variance for the simulated phenotype. 
sigma1 = .5
#The alpha parameter for the elastic net estimator when the bandwidth parameters are estimated.
alpha1 = 1/3; 


#Simulate the phenotype vector based on the following interaction model:
y1 <- expset[,16]*expset[,100]*expset[,202] + rnorm(200,0,sigma1)

####PHE-METHOD######
#Creation of all possible pairwise differences between individuals.
HEset <- cbind(y1,expset)
HEset <- HEset[order(HEset[,1]),]
foo <- t(HEset)
cols <- combn( ncol(foo), 2)
Pairdiff <- apply( cols, 2, function(z) rowDiffs(foo[,z]))
HEset <- t(Pairdiff)
# Pairwise differences of phenotype values between all individuals.
HEpheno2 <- HEset[,1]
# Pairwise differences of genes between all individuals.
HEexpset <- HEset[,-1]
# Creation of pseudo-explanatory variables with the gamma-parameter equal to 0.2 in the powered exponential kernel.
v1 <- (abs(HEexpset))^0.2
pred1 <- (-v1); 

# Inverse bandwidth parameters are estimated with the glmnet function.
fit1=glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
# 10-fold cross-validation.
cv.fit1= cv.glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
# Estimated inverse bandiwdth parameters with respect to the cross-validation based lambda-value used in the simulation section of the paper.
a1 <- (abs(coef(fit1, s = 3*cv.fit1$lambda.1se))[-1])

#Plot the results.  
plot(a1, type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("Estimated inverse bandwidth parameters")))
abline(v = c(16,100,202), col ="red", lty = 3)
