#This code is for the Dream-challenge acute myeloid leukemia data (see the reference Noren et al. 2016).
#Data can be downloaded for open research from the DREAM-website upon registration.
#Check that the dimension equals to (191,272) and remove the non-gene-expression variables.
data <- data[,c(42:272)]

#Install dependencies
library(matrixStats)
library(glmnet)


#Set residual variance for the simulated phenotypes. The following residual variances were used in the first simulation section.
sigma1 = 0.65; sigma2 = 1.05; sigma3 =1.3
#Set the alpha1 and (and lambda if the cross-validation based value is not used) for the elastic net estimator when the bandwidth parameters are estimated.
alpha1 = 1/3; #lambda = 0.18

#The number of simulated replicates
nrep = 100


#An optional vectors for the phenotype replicates
y1 <- matrix(0,nrow(data),nrep);y2 <- matrix(0,nrow(data),nrep);y3 <- matrix(0,nrow(data),nrep)
#Vector for heritabilities over replications
heritvec1 <- rep(0,nrep);heritvec2 <- rep(0,nrep);heritvec3 <- rep(0,nrep)

####Simulate replicates with average heritabilities approximately equal to 0.5, 0.3, 0.2#####

for(i in 1:nrep){
  
  expset <- data
  #simulate the phenotypes for each replicate
  y1[,i] <-  expset[,21]*expset[,105]*expset[,207] + rnorm(191,0,sigma1)
  y2[,i]<-  expset[,21]*expset[,105]*expset[,207] + rnorm(191,0,sigma2)
  y3[,i]<-  expset[,21]*expset[,105]*expset[,207] + rnorm(191,0,sigma3)
  
  heritvec1[i] <- (var(y1[,i])-sigma1^2)/var(y1[,i])
  heritvec2[i] <- (var(y2[,i])-sigma2^2)/var(y2[,i])
  heritvec3[i] <- (var(y3[,i])-sigma3^2)/var(y3[,i])
}

#####Compute the average heritabilities over replicates.
mean(heritvec1);mean(heritvec2);mean(heritvec3)

#Empty vectors for the results (estimated inverse bandwidth parameters).
PHEy1pred1 <- matrix(0,ncol(data),nrep);PHEy1pred2<- matrix(0,ncol(data),nrep);PHEy1pred3<- matrix(0,ncol(data),nrep)

# Estimation process for the replicates corresponding to the heritability levels of 0.5
for(i in 1:nrep){
  
  HEset <- cbind(y1[,i],expset)
  HEset <- HEset[order(HEset[,1]),]
  foo <- t(HEset)
  cols <- combn( ncol(foo), 2)
  tripleSums <- apply( cols, 2, function(z) rowDiffs(foo[,z]))
  HEset <- t(tripleSums)
  # Pairwise differences of phenotype values between all individuals
  HEpheno2 <- HEset[,1]
  # Pairwise differences of covariates between all individuals
  HEexpset <- HEset[,-1]
  # Different gamma parameters in the powered exponential kernel
  v1 <- (abs(HEexpset))^0.2
  v2 <- (abs(HEexpset))^1
  v3 <- (abs(HEexpset))^2
  # Creation of pseudo-explanatory variables
  pred1 <- (-v1);  pred2 <- (-v2); pred3 <- (-v3)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit1=glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit1= cv.glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a1 <- (abs(coef(fit1, s = 3*cv.fit1$lambda.1se))[-1])
  PHEy1pred1[,i] <- as.vector(a1)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit2=glmnet(pred2, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit2= cv.glmnet(pred2, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a2 <- (abs(coef(fit2, s = 3*cv.fit2$lambda.1se))[-1])
  PHEy1pred2[,i] <- as.vector(a2)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit3=glmnet(pred3, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit3= cv.glmnet(pred3, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a3 <- (abs(coef(fit3, s = 3*cv.fit3$lambda.1se))[-1])
  PHEy1pred3[,i] <- as.vector(a3)
  
  
}




######################################################





#Empty vectors for the results 
PHEy2pred1 <- matrix(0,ncol(data),nrep)

PHEy2pred2<- matrix(0,ncol(data),nrep)

PHEy2pred3<- matrix(0,ncol(data),nrep)

# Estimation process for the replicates corresponding to the heritability levels of 0.5
for(i in 1:nrep){
  
  HEset <- cbind(y2[,i],expset)
  HEset <- HEset[order(HEset[,1]),]
  foo <- t(HEset)
  cols <- combn( ncol(foo), 2)
  tripleSums <- apply( cols, 2, function(z) rowDiffs(foo[,z]))
  HEset <- t(tripleSums)
  # Pairwise differences of phenotype values between all individuals
  HEpheno2 <- HEset[,1]
  # Pairwise differences of covariates between all individuals
  HEexpset <- HEset[,-1]
  # Different gamma parameters in the powered exponential kernel
  v1 <- (abs(HEexpset))^0.2
  v2 <- (abs(HEexpset))^1
  v3 <- (abs(HEexpset))^2
  # Creation of pseudo-explanatory variables
  pred1 <- (-v1);  pred2 <- (-v2); pred3 <- (-v3)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit1=glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit1= cv.glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a1 <- (abs(coef(fit1, s = 3*cv.fit1$lambda.1se))[-1])
  PHEy2pred1[,i] <- as.vector(a1)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit2=glmnet(pred2, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit2= cv.glmnet(pred2, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a2 <- (abs(coef(fit2, s = 3*cv.fit2$lambda.1se))[-1])
  PHEy2pred2[,i] <- as.vector(a2)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit3=glmnet(pred3, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit3= cv.glmnet(pred3, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a3 <- (abs(coef(fit3, s = 3*cv.fit3$lambda.1se))[-1])
  PHEy2pred3[,i] <- as.vector(a3)
  
  
}




######################################################





#Empty vectors for the results 
PHEy3pred1 <- matrix(0,ncol(data),nrep)
PHEy3pred2<- matrix(0,ncol(data),nrep)

PHEy3pred3<- matrix(0,ncol(data),nrep)

# Estimation process for the replicates corresponding to the heritability levels of 0.5
for(i in 1:nrep){
  
  HEset <- cbind(y3[,i],expset)
  HEset <- HEset[order(HEset[,1]),]
  foo <- t(HEset)
  cols <- combn( ncol(foo), 2)
  tripleSums <- apply( cols, 2, function(z) rowDiffs(foo[,z]))
  HEset <- t(tripleSums)
  # Pairwise differences of phenotype values between all individuals
  HEpheno2 <- HEset[,1]
  # Pairwise differences of covariates between all individuals
  HEexpset <- HEset[,-1]
  # Different gamma parameters in the powered exponential kernel
  v1 <- (abs(HEexpset))^0.2
  v2 <- (abs(HEexpset))^1
  v3 <- (abs(HEexpset))^2
  # Creation of pseudo-explanatory variables
  pred1 <- (-v1);  pred2 <- (-v2); pred3 <- (-v3)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit1=glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit1= cv.glmnet(pred1, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a1 <- (abs(coef(fit1, s = 3*cv.fit1$lambda.1se))[-1])
  PHEy3pred1[,i] <- as.vector(a1)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit2=glmnet(pred2, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit2= cv.glmnet(pred2, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a2 <- (abs(coef(fit2, s = 3*cv.fit2$lambda.1se))[-1])
  PHEy3pred2[,i] <- as.vector(a2)
  
  # Inverse bandwidth parameters are estimated with glmnet function
  fit3=glmnet(pred3, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  # 10-fold cross-validation for glmnet
  cv.fit3= cv.glmnet(pred3, log(HEpheno2^2), alpha =  1/3, maxit = 10000 )
  
  a3 <- (abs(coef(fit3, s = 3*cv.fit3$lambda.1se))[-1])
  PHEy3pred3[,i] <- as.vector(a3)
  
  
}

plot(rowSums(PHEy1pred1)/max(rowSums(PHEy1pred1)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("A1      ","Heritability", "=0.50, ", gamma,"=0.2")))
abline(v = c(21,105,207), col ="red", lty = 3)
plot(rowSums(PHEy2pred1)/max(rowSums(PHEy2pred1)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("A2      ","Heritability", "=0.30, ", gamma,"=0.2")))
abline(v = c(21,105,207), col ="red", lty = 3)
plot(rowSums(PHEy3pred1)/max(rowSums(PHEy3pred1)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("A3      ","Heritability", "=0.20, ", gamma,"=0.2")))
abline(v = c(21,105,207), col ="red", lty = 3)


plot(rowSums(PHEy1pred2)/max(rowSums(PHEy1pred2)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("B1      ","Heritability", "=0.50, ", gamma,"=1")))
abline(v = c(21,105,207), col ="red", lty = 3)
plot(rowSums(PHEy2pred2)/max(rowSums(PHEy2pred2)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("B2      ","Heritability", "=0.30, ", gamma,"=1")))
abline(v = c(21,105,207), col ="red", lty = 3)
plot(rowSums(PHEy3pred2)/max(rowSums(PHEy3pred2)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("B3      ","Heritability", "=0.20, ", gamma,"=1")))
abline(v = c(21,105,207), col ="red", lty = 3)


plot(rowSums(PHEy1pred3)/max(rowSums(PHEy1pred3)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("C1      ","Heritability", "=0.50, ", gamma,"=2")))
abline(v = c(21,105,207), col ="red", lty = 3)
plot(rowSums(PHEy2pred3)/max(rowSums(PHEy2pred3)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("C2      ","Heritability", "=0.30, ", gamma,"=2")))
abline(v = c(21,105,207), col ="red", lty = 3)
plot(rowSums(PHEy3pred3)/max(rowSums(PHEy3pred3)), type = "l", xlab = "Genes", ylab = "Inverse bandwidths",main = expression(paste("C3      ","Heritability", "=0.20, ", gamma,"=2")))
abline(v = c(21,105,207), col ="red", lty = 3)



